#!/usr/bin/perl
# moniker.pl - monitors named server ports
# by Jonathan Eisenzopf. v0.2 20000107
# Copyright (c) 2000 Jonathan Eisenzopf. 
# All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Originally published and documented at http://www.motherofperl.com
# Contact eisen@pobox.com for all other uses.

# INCLUDES
use strict;
use Net::Telnet;
use Mail::Sendmail;
use Getopt::Std;
use Text::CSV_XS;

# CONSTANTS
my $timeout = 5;
my $monikeraddr = 'support@wwind.com';
my $smtpserver = 'core.iconnet.net';
my $_ERR = '';

my $services = {
    ftp    => { port => 21,   print => "", waitfor => '/220/' },
    ssh    => { port => 22,   print => "", waitfor => '/SSH/' },
    telnet => { port => 23,   print => "", waitfor => '/(login|username)/' },
    domain => { port => 42,   print => "", waitfor => '' },
    www    => { port => 80,   print => "HEAD / HTTP/1.0\n\n", waitfor => '/200/' },
    pop    => { port => 110,  print => "", waitfor => '/\+OK/' },
    nntp   => { port => 119,  print => "", waitfor => '/200/' },
    imap   => { port => 143,  print => "", waitfor => '/OK/' },
    irc    => { port => 6667, print => "", waitfor => '' },
    smtp   => { port => 25,   print => "", waitfor => '/SMTP/' }
};

# MAIN
# get the command line arguments
my %opts = ();
getopt('hsef', \%opts);

# if they passed us a filename
if (exists($opts{f}) && -e $opts{f}) {
    # create new Text::CSV object
    my $csv = new Text::CSV_XS;

    # open the file they specified
    open(FILE,$opts{f}) || die "Cannot open file $opts{f}: $!\n";
    while (<FILE>) {
	# skip the line if it's empty
	next unless /\S+/;

	# get the columns for the line
	$csv->parse($_);
	my ($host,$service,$email) = $csv->fields;
	next unless ($host =~ /\S+/ && $service =~ /\S+/);

	# check the service
	if (&check_service($host,$service)) {
	    print "$host:$service is operational.\n";
	} else {
	    &alert($email,$host,$service,$_ERR);
	}
    }
    close(FILE);
} else {
    &usage unless exists($opts{h}) && exists($opts{s});

    my $service = $opts{s};
    my $host    = $opts{h};
    my $email   = $opts{e} if exists $opts{e};

    # check the service
    if (&check_service($host,$service)) {
	print "$host:$service is operational.\n";
    } else {
	&alert($email,$host,$service,$_ERR);
    }
}

# SUBROUTINES
sub alert {
    my ($email,$host,$service,$error) = @_;
    my $port    = $services->{$service}->{port};
    my $message = "$host:$port $error\n";
    
    print "$message";
    if ($email) {
	sendmail(To      => $email,
		 Subject => 'Moniker Alert',
		 From    => $monikeraddr,
		 Message => $message,
		 Server  => $smtpserver,
		 delay   => 1,
		 retries => 5) 
	    || die "Cannot send mail: $Mail::Sendmail::error\n";
    }
}

sub check_service {
    my ($host,$service) = @_;

    # make sure user specified a valid service
    die "Service $service does not compute.\n" 
	unless defined $services->{$service};
    
    # get ready
    my $port    = $services->{$service}->{port};
    my $sock    = new Net::Telnet (Telnetmode => 0);
    my $waitfor = $services->{$service}->{waitfor};
    
    # Create a connection to the remote host
    eval {
	$sock->open(Host    => $host,
		    Port    => $port,
		    Timeout => $timeout
		    );
    };

    # Catch any errors
    if ($@) {
	$_ERR = "Cannot establish connection to host";
	return 0;
    }
    
    # send data to remote host
    $sock->print($services->{$service}->{print}) if $services->{$service}->{print};

    # wait for regex match
    if (!($sock->waitfor($waitfor))) {
	$_ERR = "Timed out waiting for $waitfor";
	return 0;
    }
    
    # things went well
    return 1;
}

sub usage {
    die <<DIE;
Usage: moniker -h host -s service [-e email]
       moniker -f <filename>
DIE
}
